var Helper = function() {
    var messagePopup = $('#popup'),
        popupVisible = messagePopup.hasClass('hidden'),
        progressContainer = $('#progress-container'),
        imageProgress = $('#image-progress'),
        progressSpan = $('#texturePath');

    return {
        initializeShaders: function(url, callback) {
            $.ajax({
                url: url,
                error: function() {
                    console.error('failed to load shader file from: ' + url);
                },
                success: callback
            });
        },

        downloadImage: function (texturePath, textureIndex, spriteIndex, whileInit) {
            me = this;
            onProgressHolder = me.showProgress(texturePath);
            if(!texturePath.endsWith('/')) {
                texturePath += '/';
            }

            var xmlHttp = new XMLHttpRequest();
            xmlHttp.open('GET', texturePath, true);
            xmlHttp.onprogress = onProgressHolder.onProgressFunction;
            xmlHttp.onload = function(e) {
                var blob = new Blob([xmlHttp.response], {
                    type: this.getResponseHeader('content-type')
                });

                var blobSrc = window.URL.createObjectURL(blob);

                var image = new Image();
                var texture = new THREE.Texture(image);

                image.onload = function() {
                    me.hideProgress();

                    texture.needsUpdate = true;
                    var params = {
                        textureIndex: textureIndex,
                        spriteIndex: spriteIndex,
                        texture: texture
                    };

                    if(textureIndex == 0 && whileInit) {
                        $(document).trigger('firstImageOnLoad', params);
                    } else {
                        $(document).trigger('imageOnLoad', params);
                    }
                }

                image.src = blobSrc;
            };

            xmlHttp.responseType = 'blob';
            xmlHttp.send();
        },

        showProgress: function(texturePath) {
            progressContainer.removeClass('hidden');
            progressSpan.text(texturePath);

            return {
                onProgressFunction: function(e) {
                    if (e.lengthComputable) {
                        value = Math.floor(e.loaded / e.total * 100);
                        imageProgress.val(value);
                    }
                }
            }
        },

        hideProgress: function() {
            imageProgress.val(0);
            progressSpan.text('');
            progressContainer.addClass('hidden');
        },

        displayMessage: function(message, fadeOutAutomatically, callback, timeout) {
            var me = this;
            messagePopup.html(message);
            messagePopup.show();

            if(fadeOutAutomatically) {
                this.fadeoutMessage(callback, timeout);
            }
        },

        fadeoutMessage: function(callback, timeout) {
            if(timeout == undefined) {
                timeout = 0;
            }

            setTimeout(function() {
                messagePopup.fadeOut(400, function() {
                    $(this).html('');
                    if(callback) {
                        callback();
                    }
                });
            }, timeout);

        }


    };
}
