var OverviewHelper = function() {
    var sceneOveriew, rendererOverview, cameraOverview, innerCubeOverview;

    //private method
    var addInnerCubeOverView = function(x, y, z) {
        var innerCubeGeometry = new THREE.BoxGeometry( x, y, z );
        var innerCubeMaterial = new THREE.MeshBasicMaterial( {color: 0xaaaaaa} );
        innerCubeOverview = new THREE.Mesh( innerCubeGeometry, innerCubeMaterial );
        sceneOverview.add( innerCubeOverview );
    }

    var initiateEventListener = function(containerOverviewElement) {
        // change overview box
        $(document).on('zoomActionFinished', function(e, params) {

            zoomedIn = params.zoomAction == 'zoomIn' || zoomedInfo != null;
            sceneOverview.remove(innerCubeOverview);

            if(zoomedIn) {
                xWidth = zoomedInfo.dimension.xywidth / textureInfo.originalSize.sizex;
                yWidth = zoomedInfo.dimension.xywidth / textureInfo.originalSize.sizey;
                zWidth = zoomedInfo.numberOfSlices / textureInfo.originalSize.sizez;

                xOffset = zoomedInfo.dimension.xmin / textureInfo.originalSize.sizex;
                yOffset = zoomedInfo.dimension.ymin / textureInfo.originalSize.sizey;
                zOffset = zoomedInfo.dimension.zmin / textureInfo.originalSize.sizez;

                addInnerCubeOverView(xWidth, yWidth, zWidth);

                // main cube was rotated around x axis, so have to swap y and z coordinates
                // for the overview
                innerCubeOverview.position.x = -0.5 + xOffset + xWidth / 2;
                innerCubeOverview.position.y = (-0.5 + zOffset + zWidth / 2);
                innerCubeOverview.position.z = (-0.5 + yOffset + yWidth / 2) * -1;
                //innerCubeOverview.applyMatrix((new THREE.Matrix4()).makeRotationX(Math.PI / -2))
            } else {
                addInnerCubeOverView(1,1,1);
            }
        });
        var headerHeight = $('header').height();
        $(window).on('scroll', function(e) {
            $containerOverviewElement = $(containerOverviewElement)
            if($(this).scrollTop() >= headerHeight) {
                $containerOverviewElement.css('position', 'relative');
                $containerOverviewElement.css('top', $(this).scrollTop() - headerHeight);
            } else {
                $containerOverviewElement.css('top', 0);
            }
        });
    }

    var addTextsToScene = function() {
        var shapeParameters = {font: 'helvetiker', size: 0.2, height: 0.05},
            shape = new THREE.TextGeometry("0", shapeParameters),
            wrapper = new THREE.MeshBasicMaterial({color: 0x000000}),
            words = new THREE.Mesh(shape, wrapper);

        words.position.x = -0.6;
        words.position.y = -0.6;
        words.position.z = 0.52;
        sceneOverview.add(words);
    }

    var addCubeEdges = function() {
        // fix for cube
        var numberOfEdges = 12;
        var xColorEdge = new THREE.Color(1,0,0);
        var yColorEdge = new THREE.Color(0,1,0);
        var zColorEdge = new THREE.Color(0,0,1);

        var geometry = new THREE.BufferGeometry();
        var material = new THREE.LineBasicMaterial({ vertexColors: THREE.VertexColors });

        var colors = new Float32Array( numberOfEdges * 2 * 3 );
        var positions = new Float32Array([
            // edges in x direction
            // front face
            -0.5, -0.5, -0.5,  0.5, -0.5, -0.5,
            -0.5,  0.5, -0.5,  0.5,  0.5, -0.5,
            // back face
            -0.5, -0.5,  0.5,  0.5, -0.5,  0.5,
            -0.5,  0.5,  0.5,  0.5,  0.5,  0.5,

            // edges in y direction
            // front face
            -0.5, -0.5, -0.5, -0.5,  0.5, -0.5,
             0.5, -0.5, -0.5,  0.5,  0.5, -0.5,
            // back face
            -0.5, -0.5,  0.5, -0.5,  0.5,  0.5,
             0.5, -0.5,  0.5,  0.5,  0.5,  0.5,

            // edges in z direction
            // left side
            -0.5, -0.5, -0.5, -0.5, -0.5,  0.5,
            -0.5,  0.5, -0.5, -0.5,  0.5,  0.5,
            // right side
             0.5, -0.5, -0.5,  0.5, -0.5,  0.5,
             0.5,  0.5, -0.5,  0.5,  0.5,  0.5
        ]);

        for(edgeCounter = 0; edgeCounter < 12 * 2; edgeCounter++) {
            var colorToUse = null
            if(edgeCounter <= 7) {
                colorToUse = xColorEdge;
            } else if(edgeCounter <= 15) {
                colorToUse = zColorEdge;
            } else {
                colorToUse = yColorEdge;
            }

            colors[edgeCounter * 3 + 0] = colorToUse.r;
            colors[edgeCounter * 3 + 1] = colorToUse.g;
            colors[edgeCounter * 3 + 2] = colorToUse.b;
        }

        geometry.addAttribute( 'position', new THREE.BufferAttribute( positions, 3 ) );
        geometry.addAttribute( 'color', new THREE.BufferAttribute( colors, 3 ) );

        var lines = new THREE.Line( geometry, material, THREE.LinePieces );
        sceneOverview.add( lines );
    }

    return {
        initOverview: function(containerOverviewElement) {
            sceneOverview = new THREE.Scene();
            cameraOverview = new THREE.PerspectiveCamera( 45, 150 / 150, .1, 1000 );
            cameraOverview.position.z = 2;

            var cubeGeometry = new THREE.BoxGeometry( 1, 1, 1 );
            var cubeMaterial = new THREE.MeshBasicMaterial( {color: 0x000000} );
            var cubeOverview = new THREE.Mesh( cubeGeometry, cubeMaterial );
            var edgesHelper = new THREE.EdgesHelper(cubeOverview, 0x000000);
            edgesHelper.material.linewidth = 2;
            //sceneOverview.add( edgesHelper );

            addInnerCubeOverView(1,1,1);
            addCubeEdges();
            addTextsToScene();

            rendererOverview = new THREE.WebGLRenderer();
            rendererOverview.setSize(200, 200);
            rendererOverview.setClearColor(0xffffff, 1);
            containerOverviewElement.appendChild(rendererOverview.domElement);

            initiateEventListener(containerOverviewElement);
        },

        updateCameraPosition: function(position) {
            cameraPoint.normalize().multiplyScalar(4);
            cameraPoint = sceneOverview.position.clone().add(cameraPoint);

            cameraOverview.position.x = cameraPoint.x;
            cameraOverview.position.y = cameraPoint.y;
            cameraOverview.position.z = cameraPoint.z;
            cameraOverview.lookAt(sceneOverview.position);
        },

        render: function() {

            rendererOverview.render(sceneOverview, cameraOverview);
        }
    }
}
