#ifdef GL_ES
precision highp float;
#endif

uniform sampler2D tBackground;
uniform sampler2D tvSlices[10];
uniform sampler2D tTransfer;

uniform float numberOfSprites;
uniform float slicesPerSprite;

uniform float numberOfSlices;
uniform float slicesOverX;
uniform float slicesOverY;
uniform float steps;

varying vec4 vProjPos;
varying vec4 vFrontColor;
varying vec2 vUv;

const float maxSteps = 200.0;

float numberOfSlicePositions;

float getVolumeValue(vec3 volpos)
{
    float s1Original,s2Original, s1,s2;
    float dx1,dy1;
    float dx2,dy2;
    float value;

    vec2 texpos1,texpos2;
    vec4 color;

    // use the numberOfSlices to stretch volume
    // over black slices in the end
    s1Original = floor(volpos.z * numberOfSlices);
    s2Original = s1 + 1.0;

    int tex1Index = int(floor(s1Original / slicesPerSprite));
    int tex2Index = int(floor(s2Original / slicesPerSprite));

    // number of z layer in sprite part
    s1 = mod(s1Original, slicesPerSprite);
    s2 = mod(s2Original, slicesPerSprite);

    dx1 = fract(s1/slicesOverX);
    dy1 = floor(s1/slicesOverY)/slicesOverY;

    dx2 = fract(s2/slicesOverX);
    dy2 = floor(s2/slicesOverY)/slicesOverY;

    texpos1.x = dx1+(volpos.x/slicesOverX);
    texpos1.y = dy1+(volpos.y/slicesOverY);

    texpos2.x = dx2+(volpos.x/slicesOverX);
    texpos2.y = dy2+(volpos.y/slicesOverY);

    texpos1.y = 1.0 - texpos1.y;
    texpos2.y = 1.0 - texpos2.y;

    float value1, value2;
    bool value1Set = false, value2Set = false;

    for (int x = 0; x < 10; x++) {
        if(x == tex1Index) {
            value1 = texture2D(tvSlices[x],texpos1).x;
            value1Set = true;
        }

        if(x == tex2Index) {
            value2 = texture2D(tvSlices[x],texpos2).x;
            value2Set = true;
        }

        if(value1Set && value2Set) {
            break;
        }
    }

    if(s2 >= numberOfSlices) {
        value = value1;
    } else {
        value = value1;
    }

    return value;
}

void main(void)
{
    numberOfSlicePositions = numberOfSprites * slicesPerSprite;

    vec2 texC = vProjPos.xy/vProjPos.w;
    texC.x = 0.5*texC.x + 0.5;
    texC.y = 0.5*texC.y + 0.5;

    vec4 backColor = texture2D(tBackground,texC);

    vec4 frontColor = vFrontColor;
    vec3 dir = backColor.rgb - frontColor.rgb;
    vec4 vpos = frontColor;

    float cont = 0.0;

    vec3 Step = dir/steps;

    vec4 accum = vec4(0, 0, 0, 0);
    vec4 sample = vec4(0.0, 0.0, 0.0, 0.0);
    vec4 value = vec4(0, 0, 0, 0);

    float opacityFactor = 8.0;
    float lightFactor = 1.3;
    vec2 tf_pos;
    for(float i = 0.0; i < maxSteps; i+=1.0)
    {
        if(i == steps) {
            break;
        }

        tf_pos.x = getVolumeValue(vpos.xyz);
        tf_pos.y = 0.5;

        value = texture2D(tTransfer,tf_pos);

        if(value.a < 0.15) {
            value = vec4(0.0);
        }

        // Process the volume sample
        sample.a = value.a * opacityFactor * (1.0 / steps);
        sample.rgb = value.rgb * sample.a * lightFactor;

        accum.rgb += (1.0 - accum.a) * sample.rgb;
        accum.a += sample.a;

        //advance the current position
        vpos.xyz += Step;

        //break if the position is greater than <1, 1, 1>
        if(vpos.x > 1.0 || vpos.y > 1.0 || vpos.z > 1.0) {
          //accum = vFrontColor;
          break;
        }

        if(accum.a >= 0.95) {
          //accum = vFrontColor;
          break;
        }
    }
    gl_FragColor =  accum;
    //gl_FragColor = vFrontColor;
}
