import os

from skimage import img_as_ubyte

from multiprocessing import Process, cpu_count, Queue

import logging

from django.conf import settings

from bson.objectid import ObjectId

import math
import numpy as np

logger = logging.getLogger(__name__)


def multiprocess_img_as_ubyte(frames):
    logger.debug('using single process implementation')
    return img_as_ubyte(frames)

    pathMmap = settings.PATH_MMAP + 'tmp/'

    if os.path.exists(pathMmap) is False:
        os.makedirs(pathMmap)

    logger.debug('converting %d frames to ubyte' % frames.shape[0])
    cpus = cpu_count()
    inputQueue = Queue()

    imagesPerProcess = frames.shape[1]
    onlyOneFrame = frames.shape[0] is 1

    if onlyOneFrame:
        imagesPerProcess = int(math.ceil(frames.shape[1] / cpus))

    initialNumberOfContainers = 0

    fd = np.memmap(pathMmap + str(ObjectId()), mode='w+', shape=frames.shape, dtype=frames.dtype)
    fd[:] = frames[:]
    outputFd = np.memmap(pathMmap + str(ObjectId()), mode='w+', shape=frames.shape, dtype=np.uint8)

    for frameNumber in range(0, frames.shape[0]):
        if onlyOneFrame:
            for i in range(0, cpus):
                container = FrameContainer(fd, outputFd, frameNumber, imagesPerProcess * i, imagesPerProcess, onlyOneFrame)
                inputQueue.put(container)
                initialNumberOfContainers += 1
        else:
            container = FrameContainer(fd, outputFd, frameNumber, 0, imagesPerProcess, onlyOneFrame)
            inputQueue.put(container)
            initialNumberOfContainers += 1

    logger.debug('%d containers for jobs created' % initialNumberOfContainers)
    logger.debug('creating output queues and processes')
    queues = [Queue() for i in range(0, cpus)]
    processes = [Process(target=worker_img_as_ubyte, args=(queues[i],)) for i in range(0, cpus)]

    logger.debug('starting processes')
    for process in processes:
        process.start()

    for process in processes:
        process.join()

    newFrames = np.empty(frames.shape, dtype=np.uint8)
    newFrames[:] = outputFd[:]

    del frames

    return newFrames


class FrameContainer():
    def __init__(self, inputFd, outputFd, frameNumber, beginNumber, numberOfImages, onlyOneFrame):
        self.inputFd = inputFd
        self.outputFd = outputFd
        self.frameNumber = frameNumber
        self.beginNumber = beginNumber
        self.numberOfImages = numberOfImages
        self.onlyOneFrame = onlyOneFrame


def worker_img_as_ubyte(inputQueue):
    while inputQueue.empty() is False:
        container = inputQueue.get()
        logger.debug('img_as_ubyte frameNumber %d' % container.frameNumber)

        frame_number = container.frameNumber
        begin_number = container.beginNumber
        number_of_images = container.numberOfImages
        frame = container.frame

        container.outputFd[frame_number][begin_number: begin_number + number_of_images] = img_as_ubyte(frame[begin_number: begin_number + number_of_images])
